@@ -1,7 +1,7 @@
 /*
  * gpe-package
  *
- * Copyright (C) 2003  Florian Boor <florian.boor@kernelconcepts.de>
+ * Copyright (C) 2003,2004  Florian Boor <florian.boor@kernelconcepts.de>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -22,8 +22,10 @@
 #include <poll.h>
 #include <sys/types.h>
 #include <sys/socket.h>
+#include <sys/signal.h>
 #include <sys/un.h>
 
+#include <locale.h>
 #include <libintl.h>
 #define _(x) gettext(x)
 
@@ -35,6 +37,7 @@
 #include <gpe/errorbox.h>
 #include <gpe/spacing.h>
 #include <gpe/pixmaps.h>
+#include <gpe/gpehelp.h>
 
 #include "packages.h"
 #include "interface.h"
@@ -43,17 +46,45 @@
 
 #define N_(x) (x)
 
+#define MI_FILE             1
+#define MI_FILE_INSTALL     2
+#define MI_FILE_CLOSE       3
+#define MI_PACKAGES         4
+#define MI_PACKAGES_UPDATE  5
+#define MI_PACKAGES_UPGRADE 6
+#define MI_PACKAGES_APPLY   7
+#define MI_FILTER_INST		8
+#define MI_FILTER_NOTINST	9
+#define MI_PACKAGES_INFO    10
+
+#define TREE_SHOW_ALL		0x00
+#define TREE_SHOW_INST		0x01
+#define TREE_SHOW_NOTINST	0x02
+
+#define HELPMESSAGE "GPE-Package\nVersion " VERSION \
+		"\nGPE frontend for ipkg\n\nflorian@handhelds.org"
+
+#define NOHELPMESSAGE N_("Displaying help failed.")
+
 /* --- module global variables --- */
-void create_fMain (void);
-void on_select_local(GtkButton *button, gpointer user_data);
-void on_packages_update_clicked(GtkButton *button, gpointer user_data);
-void on_system_update_clicked(GtkButton *button, gpointer user_data);
-void on_package_install_clicked(GtkButton *button, gpointer user_data);
+
+
+typedef struct
+{
+	char *name;
+	char *version;
+	char *description;
+	char *color;
+	pkg_state_status_t status;
+}
+description_t;
+
+static description_t *pkg_info = NULL;
+static int pkg_count = 0;
+static int tree_filter = TREE_SHOW_ALL;
 
 int sock;
 static pkcommand_t running_command = CMD_NONE;
-static char *last_pkgname = NULL;
-static GtkTreeIter last_pkgiter;
 static int pkg_selection_changed = 0;
 static int error = 0;
 
@@ -63,30 +94,52 @@ static GtkWidget *txLog;
 static GtkWidget *treeview;
 static GtkTreeStore *store = NULL;
 static GtkWidget *bApply;
-//*bUpdate , *bSysUpgrade, *bSelectLocal;
+static GtkWidget *miUpdate, *miSysUpgrade, *miSelectLocal, *miApply;
+static GtkWidget *miFilterInst, *miFilterNotInst;
 static GtkWidget *sbar;
 static GtkWidget *fMain;
 static GtkWidget *dlgAction = NULL;
+static GtkWidget *dlgInfo = NULL;
+static GtkTextBuffer *infobuffer = NULL;
 static GtkWidget *mMain;
 
 
+/* some forwards */
+gboolean get_pending_messages ();
+void on_tree_filter_changed(GtkCheckMenuItem *menuitem, gpointer user_data);
+void create_fMain (void);
+void on_select_local(GtkButton *button, gpointer user_data);
+void on_packages_update_clicked(GtkButton *button, gpointer user_data);
+void on_system_update_clicked(GtkButton *button, gpointer user_data);
+void on_package_install_clicked(GtkButton *button, gpointer user_data);
+void on_about_clicked (GtkWidget * w);
+void on_help_clicked (GtkWidget * w);
+void on_package_info_clicked(GtkButton *button, gpointer user_data);
+
+
 static GtkItemFactoryEntry mMain_items[] = {
-  { N_("/_File"),         NULL,         NULL,           0, "<Branch>" },
-  { N_("/File/_Install file"), "", on_select_local,    0, "<Item>"},
+  { N_("/_File"),         NULL,         NULL, MI_FILE, "<Branch>" },
+  { N_("/File/_Install file"), "", on_select_local, MI_FILE_INSTALL, "<Item>"},
   { N_("/_File/s1"), NULL , NULL,    0, "<Separator>"},
-  { N_("/File/_Close"),  NULL, do_safe_exit,    0, "<StockItem>", GTK_STOCK_QUIT },
-  { N_("/_Packages"),         NULL,         NULL,           0, "<Branch>" },
-  { N_("/Packages/_Update lists"), "<Control> U", on_packages_update_clicked,    0, "<Item>"},
-  { N_("/Packages/Upgrade _System"), "", on_system_update_clicked,    0, "<Item>"},
+  { N_("/File/_Close"),  NULL, do_safe_exit, MI_FILE_CLOSE, "<StockItem>", GTK_STOCK_QUIT },
+  { N_("/_Packages"),         NULL,         NULL, MI_PACKAGES, "<Branch>" },
+  { N_("/Packages/Show _Installed"), "", on_tree_filter_changed, MI_FILTER_INST , "<CheckItem>"},
+  { N_("/Packages/Show _Not Installed"), "", on_tree_filter_changed, MI_FILTER_NOTINST, "<CheckItem>"},
   { N_("/_Packages/s2"), NULL , NULL,    0, "<Separator>"},
-  { N_("/Packages/_Apply"), "", on_package_install_clicked,    0, "<StockItem>", GTK_STOCK_APPLY},
+  { N_("/Packages/Show _Info"), "<Control> I", on_package_info_clicked, MI_PACKAGES_INFO , "<Item>"},
+  { N_("/_Packages/s3"), NULL , NULL,    0, "<Separator>"},
+  { N_("/Packages/_Update lists"), "<Control> U", on_packages_update_clicked, MI_PACKAGES_UPDATE , "<Item>"},
+  { N_("/Packages/Upgrade _System"), "", on_system_update_clicked, MI_PACKAGES_UPGRADE, "<Item>"},
+  { N_("/_Packages/s3"), NULL , NULL,    0, "<Separator>"},
+  { N_("/Packages/_Apply"), "", on_package_install_clicked, MI_PACKAGES_APPLY, "<StockItem>", GTK_STOCK_APPLY},
   { N_("/_Help"),         NULL,         NULL,           0, "<Branch>" },
-  { N_("/_Help/Index"),   NULL,         NULL,    0, "<Item>" },
-  { N_("/_Help/About"),   NULL,         NULL,    0, "<Item>" },
+  { N_("/_Help/Index"),   NULL,         on_help_clicked,    0, "<StockItem>",GTK_STOCK_HELP },
+  { N_("/_Help/About"),   NULL,         on_about_clicked,    0, "<Item>" },
 };
 
 int mMain_items_count = sizeof(mMain_items) / sizeof(GtkItemFactoryEntry);
 
+
 struct gpe_icon my_icons[] = {
   { "save" },
   { "cancel" },
@@ -99,12 +152,71 @@ struct gpe_icon my_icons[] = {
 };
 
 
-gboolean get_pending_messages ();
-
 
 /* dialogs */
-#warning todo panel
-#warning todo separation
+#warning todo: status icons, verbose dialogs
+
+void
+destroy_package_list(description_t *list, int len)
+{
+	while (len)
+	{
+		if (list[len-1].name) g_free(list[len-1].name);
+		if (list[len-1].description) g_free(list[len-1].description);
+		if (list[len-1].version) g_free(list[len-1].version);
+		len--;
+	}
+	if (list) g_free(list);
+}
+
+
+gboolean
+dialog_destroy()
+{
+	gtk_widget_destroy(dlgInfo);
+	dlgInfo = NULL;
+	infobuffer = NULL;
+	return TRUE;
+}
+
+
+void
+do_package_info(const char* name, const char *info)
+{
+	GtkWidget *sw;
+	GtkWidget *textview;
+	GtkTextBuffer *buffer;
+	
+	if (dlgInfo == NULL)
+	{
+		dlgInfo = gtk_dialog_new_with_buttons(_("Package information"),GTK_WINDOW(fMain),
+				   GTK_DIALOG_DESTROY_WITH_PARENT,
+				   GTK_STOCK_OK,
+				   GTK_RESPONSE_ACCEPT,
+				   NULL);
+		sw = gtk_scrolled_window_new(NULL,NULL);
+		gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(sw),
+			GTK_POLICY_AUTOMATIC,GTK_POLICY_AUTOMATIC);
+		buffer = gtk_text_buffer_new(NULL);
+		infobuffer = buffer;
+		gtk_text_buffer_insert_at_cursor(GTK_TEXT_BUFFER(buffer),info,strlen(info));
+		textview = gtk_text_view_new_with_buffer(buffer);
+		gtk_text_view_set_editable(GTK_TEXT_VIEW(textview),FALSE);
+		gtk_text_view_set_wrap_mode(GTK_TEXT_VIEW(textview),GTK_WRAP_WORD);
+		gtk_container_add(GTK_CONTAINER(sw),textview);
+		gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dlgInfo)->vbox),sw,TRUE,TRUE,0);
+		gtk_widget_set_size_request(textview,220,250);
+		gtk_widget_show_all(dlgInfo);
+		g_signal_connect_swapped(dlgInfo,"response",G_CALLBACK(dialog_destroy),NULL);
+	}
+	else
+	{
+		if (!infobuffer) return;
+		gtk_text_buffer_insert_at_cursor(GTK_TEXT_BUFFER(infobuffer),
+			info,strlen(info));
+	}
+}
+
 
 GtkWidget *
 progress_dialog (gchar * text, GdkPixbuf * pixbuf)
@@ -136,6 +248,21 @@ progress_dialog (gchar * text, GdkPixbuf * pixbuf)
 }
 
 
+void
+show_message(GtkMessageType type, char* message)
+{
+	GtkWidget* dialog;
+	
+	dialog = gtk_message_dialog_new (GTK_WINDOW(fMain),
+					 GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
+					 type,
+					 GTK_BUTTONS_OK,
+					 message);
+	gtk_dialog_run (GTK_DIALOG(dialog));
+	gtk_widget_destroy(dialog);
+}
+
+
 /* message send and receive */
 
 static void
@@ -155,7 +282,9 @@ send_message (pkcontent_t ctype, pkcommand_t command, char* params, char* list)
 		{
 			case CMD_LIST:
 				desc = _("Reading packages list...");
-				gtk_tree_store_clear(GTK_TREE_STORE(store));
+				destroy_package_list(pkg_info,pkg_count);
+				pkg_count = 0;
+				pkg_info = NULL;			
 			break;
 			case CMD_UPDATE:
 				desc = _("Updating package lists");
@@ -166,6 +295,9 @@ send_message (pkcontent_t ctype, pkcommand_t command, char* params, char* list)
 			case CMD_INSTALL:
 				desc = g_strdup_printf("%s %s", _("Installing"), list);
 			break;
+			case CMD_INFO:
+				desc = g_strdup_printf("%s", _("Retrieving package information"));
+			break;
 			default:
 				desc = _("Working...");			
 			break;
@@ -192,8 +324,6 @@ void printlog(GtkWidget *textview, gchar *str)
 }
 
 
-/* --- local intelligence --- */
-
 void 
 wait_command_finish()
 {
@@ -207,6 +337,8 @@ wait_command_finish()
 }
 
 
+/* --- local intelligence --- */
+
 gboolean
 update_check(GtkTreeModel *model, GtkTreePath *path,
              GtkTreeIter *iter,
@@ -214,8 +346,7 @@ update_check(GtkTreeModel *model, GtkTreePath *path,
 {
 	pkg_state_want_t dstate;
 	char *name;
-
-	gtk_tree_model_get (GTK_TREE_MODEL(store), iter, 
+	gtk_tree_model_get (GTK_TREE_MODEL(model), iter, 
 						COL_DESIREDSTATE, &dstate,
 						COL_NAME,&name,	-1);
 	switch (dstate)
@@ -223,7 +354,7 @@ update_check(GtkTreeModel *model, GtkTreePath *path,
 		case SW_INSTALL:
 			send_message(PK_COMMAND,CMD_INSTALL,"",name);
 			wait_command_finish();
-			gtk_tree_store_set (GTK_TREE_STORE(store), iter, 
+			gtk_tree_store_set (GTK_TREE_STORE(model), iter, 
                       COL_INSTALLED,TRUE,
                       COL_DESIREDSTATE,SW_UNKNOWN,
                       COL_COLOR,NULL,-1);
@@ -231,7 +362,7 @@ update_check(GtkTreeModel *model, GtkTreePath *path,
 		case SW_DEINSTALL:
 			send_message(PK_COMMAND,CMD_REMOVE,"",name);
 			wait_command_finish();
-			gtk_tree_store_set (GTK_TREE_STORE(store), iter, 
+			gtk_tree_store_set (GTK_TREE_STORE(model), iter, 
                       COL_INSTALLED,FALSE,
                       COL_DESIREDSTATE,SW_UNKNOWN,
                       COL_COLOR,NULL,-1);
@@ -260,59 +391,109 @@ void do_question(int nr, char *question)
 }
 
 
-void do_list(int prio,char* pkgname,char *desc, char *version, pkg_state_status_t status)
+void
+update_tree(void)
 {
+	guint id;
+	int i;
 	GtkTreeIter iter;
-	gboolean isinstalled = FALSE;
-	gboolean updatev;
-	char *color = NULL;
-	char *lversion;
-	char *nversion = NULL;
+
+	id = gtk_statusbar_get_context_id(GTK_STATUSBAR(sbar),"upd");
+	gtk_statusbar_push(GTK_STATUSBAR(sbar),
+		id,_("Updating views, please wait..."));
 	
-	updatev = (last_pkgname != NULL) && !strcmp(pkgname,last_pkgname);
+#warning improve this
+	gtk_tree_store_clear(GTK_TREE_STORE(store));
 	
-	if (updatev) 
+	for (i=0;i<pkg_count;i++)
 	{
-		iter = last_pkgiter;
-		gtk_tree_model_get(GTK_TREE_MODEL(store),&iter,
-	                       COL_VERSION, &lversion, 
-		                   COL_INSTALLED, &isinstalled, 
-		                   COL_COLOR, &color, -1);
-		nversion = g_strdup_printf("%s\n%s",lversion,version);
-		version = nversion;
+		if (((tree_filter & TREE_SHOW_INST) && (pkg_info[i].status == SS_INSTALLED))
+			|| ((tree_filter & TREE_SHOW_NOTINST) && (pkg_info[i].status != SS_INSTALLED)))		
+			continue;
+		
+		gtk_tree_store_append (store, &iter, NULL);
+	
+		gtk_tree_store_set (store, &iter,
+		    COL_NAME, pkg_info[i].name,
+		    COL_DESCRIPTION, pkg_info[i].description,
+			COL_VERSION, pkg_info[i].version,
+			COL_INSTALLED, pkg_info[i].status == SS_INSTALLED,
+			COL_DESIREDSTATE, SW_UNKNOWN,
+			COL_COLOR, pkg_info[i].color,
+	    	-1);
+	}
+	gtk_statusbar_pop(GTK_STATUSBAR(sbar),id);	
+}
+
+
+void
+on_tree_filter_changed(GtkCheckMenuItem *menuitem, gpointer user_data)
+{
+	tree_filter =
+		!gtk_check_menu_item_get_active(GTK_CHECK_MENU_ITEM(miFilterInst)) *
+		TREE_SHOW_INST +
+		!gtk_check_menu_item_get_active(GTK_CHECK_MENU_ITEM(miFilterNotInst)) *
+		TREE_SHOW_NOTINST;
+	update_tree();
+}
+
+
+void
+on_about_clicked (GtkWidget * w)
+{
+	show_message(GTK_MESSAGE_INFO,HELPMESSAGE);
+}
+
+
+void
+on_help_clicked (GtkWidget * w)
+{
+	if (gpe_show_help("gpe-package",NULL))
+		show_message(GTK_MESSAGE_ERROR,NOHELPMESSAGE);
+}
+
+
+void 
+do_list(int prio,char* pkgname,char *desc, char *version, pkg_state_status_t status)
+{
+	gboolean updatev;
+	char *nversion = NULL;
+
+	updatev = (pkg_count && !strcmp(pkgname,pkg_info[pkg_count-1].name));
+
+	if (!updatev)
+	{
+		pkg_info = realloc(pkg_info,sizeof(description_t) * (pkg_count + 1));		
+		pkg_info[pkg_count].name = g_strdup(pkgname);
+		pkg_info[pkg_count].version = g_strdup(version);
+		pkg_info[pkg_count].description = g_strdup(desc);
+		pkg_info[pkg_count].status = status;
+		pkg_info[pkg_count].color = NULL;
+		if ((status != SS_INSTALLED) && (status != SS_NOT_INSTALLED))
+			pkg_info[pkg_count].color = C_INCOMPLETE;
+		pkg_count++;
 	}
 	else
 	{
-		if (last_pkgname != NULL) 
-			g_free(last_pkgname);
-		last_pkgname = g_strdup(pkgname);
-	}
-	
-	if ((status != SS_INSTALLED) && (status != SS_NOT_INSTALLED))
-		color = C_INCOMPLETE;
-	isinstalled = isinstalled || (status==SS_INSTALLED);
+		nversion = pkg_info[pkg_count-1].version;
+		pkg_info[pkg_count-1].version = g_strdup_printf("%s\n%s",nversion,version);
+		if (pkg_info[pkg_count-1].status != SS_INSTALLED)
+			pkg_info[pkg_count-1].status = status;
+		if (nversion) g_free(nversion);
+		if ((status != SS_INSTALLED) && (status != SS_NOT_INSTALLED))
+			pkg_info[pkg_count-1].color = C_INCOMPLETE;
+	}		
 	
+		
 	switch (running_command)
 	{
 		case CMD_LIST:
-			if (!updatev)
-			{				
-				gtk_tree_store_append (store, &iter, NULL);
-				last_pkgiter = iter;
-			}
-			gtk_tree_store_set (store, &iter,
-			    COL_NAME, pkgname,
-			    COL_DESCRIPTION, desc,
-				COL_VERSION, version,
-				COL_INSTALLED, isinstalled,
-				COL_DESIREDSTATE, SW_UNKNOWN,
-				COL_COLOR, color,
-			    -1);
+		{
+		}
 		break;
 		default:
 			printlog(txLog,pkgname);
 	}
-	if (updatev) g_free(nversion);
 }
 
 
@@ -340,16 +521,18 @@ void do_info(int priority, char *str1, char *str2)
 
 void do_end_command()
 {
-#warning - change in menu
-//    gtk_widget_set_sensitive(bUpdate,TRUE);
+    gtk_widget_set_sensitive(miUpdate,TRUE);
+    gtk_widget_set_sensitive(miApply,TRUE);
+	gtk_widget_set_sensitive(miSysUpgrade,TRUE);
     gtk_widget_set_sensitive(bApply,TRUE);
-//	gtk_widget_set_sensitive(bSysUpgrade,TRUE);
     printlog(txLog,_("Command finished. Please check log messages for errors."));
 	if (dlgAction) 
 	{
 		gtk_widget_destroy(dlgAction);
 		dlgAction = NULL;
 	}
+	if (running_command == CMD_LIST) 
+		update_tree();
 	running_command = CMD_NONE;
 }
 
@@ -422,9 +605,15 @@ get_pending_messages ()
 			case PK_INFO:
 				do_info(msg.content.tf.priority, msg.content.tf.str1,msg.content.tf.str2);
 			break;	
+			case PK_PKGINFO:
+				do_package_info(msg.content.tf.str1, msg.content.tf.str2);
+			break;	
 			case PK_PACKAGESTATE:
 //				do_state(msg.content.tf.priority, msg.content.tf.str1,msg.content.tf.str2);
 			case PK_FINISHED:
+#ifdef DEBUG				
+				printf("finished\n");
+#endif			
 				do_end_command();
 			default:
 				break;
@@ -513,7 +702,9 @@ mainloop (int argc, char *argv[])
 	bind_textdomain_codeset (PACKAGE, "UTF-8");
 	textdomain (PACKAGE);
 	
-	
+	signal (SIGINT, do_safe_exit);
+	signal (SIGTERM, do_safe_exit);
+ 	
 	/* Create socket from which to read. */
 	sock = socket (AF_UNIX, SOCK_STREAM, 0);
 	if (sock < 0)
@@ -570,23 +761,42 @@ on_select_local(GtkButton *button, gpointer user_data)
 }
 
 
+void
+on_package_info_clicked(GtkButton *button, gpointer user_data)
+{
+	GtkTreeIter iter;
+	GtkTreeSelection *sel;
+	char *name = NULL;
+	
+	sel = gtk_tree_view_get_selection(GTK_TREE_VIEW(treeview));
+	
+	if (gtk_tree_selection_get_selected(sel, (GtkTreeModel**)(&store), &iter))
+	{
+		gtk_tree_model_get (GTK_TREE_MODEL(store), &iter, COL_NAME, &name, -1);
+		send_message(PK_COMMAND,CMD_INFO,name,name);
+	}
+}
+
+
 void
 on_system_update_clicked(GtkButton *button, gpointer user_data)
 {
-  GtkTextBuffer *logbuf;
-  GtkTextIter start,end;
-//  gtk_widget_set_sensitive(bUpdate,FALSE);
-  gtk_widget_set_sensitive(bApply,FALSE);
-//  gtk_widget_set_sensitive(bSysUpgrade,FALSE);
+	GtkTextBuffer *logbuf;
+	GtkTextIter start,end;
+	
+    gtk_widget_set_sensitive(miUpdate,FALSE);
+    gtk_widget_set_sensitive(miApply,FALSE);
+	gtk_widget_set_sensitive(miSysUpgrade,FALSE);
+	gtk_widget_set_sensitive(bApply,FALSE);
 
 	error = 0;
 	
-  /* clear log */	
-  logbuf = gtk_text_view_get_buffer(GTK_TEXT_VIEW(txLog));
-  gtk_text_buffer_get_start_iter(GTK_TEXT_BUFFER(logbuf),&start);
-  gtk_text_buffer_get_end_iter(GTK_TEXT_BUFFER(logbuf),&end);
-  gtk_text_buffer_delete(GTK_TEXT_BUFFER(logbuf),&start,&end);
-  send_message(PK_COMMAND,CMD_UPDATE,"","");
+	/* clear log */	
+	logbuf = gtk_text_view_get_buffer(GTK_TEXT_VIEW(txLog));
+	gtk_text_buffer_get_start_iter(GTK_TEXT_BUFFER(logbuf),&start);
+	gtk_text_buffer_get_end_iter(GTK_TEXT_BUFFER(logbuf),&end);
+	gtk_text_buffer_delete(GTK_TEXT_BUFFER(logbuf),&start,&end);
+	send_message(PK_COMMAND,CMD_UPDATE,"","");
 	wait_command_finish();
 	send_message(PK_COMMAND,CMD_UPGRADE,"","");
 	wait_command_finish();
@@ -603,20 +813,22 @@ on_system_update_clicked(GtkButton *button, gpointer user_data)
 void
 on_packages_update_clicked(GtkButton *button, gpointer user_data)
 {
-  GtkTextBuffer *logbuf;
-  GtkTextIter start,end;
-//  gtk_widget_set_sensitive(bUpdate,FALSE);
-  gtk_widget_set_sensitive(bApply,FALSE);
-//  gtk_widget_set_sensitive(bSysUpgrade,FALSE);
+	GtkTextBuffer *logbuf;
+	GtkTextIter start,end;
+
+    gtk_widget_set_sensitive(miUpdate,FALSE);
+    gtk_widget_set_sensitive(miApply,FALSE);
+	gtk_widget_set_sensitive(miSysUpgrade,FALSE);
+	gtk_widget_set_sensitive(bApply,FALSE);
 
 	error = 0;
 	
-  /* clear log */	
-  logbuf = gtk_text_view_get_buffer(GTK_TEXT_VIEW(txLog));
-  gtk_text_buffer_get_start_iter(GTK_TEXT_BUFFER(logbuf),&start);
-  gtk_text_buffer_get_end_iter(GTK_TEXT_BUFFER(logbuf),&end);
-  gtk_text_buffer_delete(GTK_TEXT_BUFFER(logbuf),&start,&end);
-  send_message(PK_COMMAND,CMD_UPDATE,"","");
+	/* clear log */	
+	logbuf = gtk_text_view_get_buffer(GTK_TEXT_VIEW(txLog));
+	gtk_text_buffer_get_start_iter(GTK_TEXT_BUFFER(logbuf),&start);
+	gtk_text_buffer_get_end_iter(GTK_TEXT_BUFFER(logbuf),&end);
+	gtk_text_buffer_delete(GTK_TEXT_BUFFER(logbuf),&start,&end);
+	send_message(PK_COMMAND,CMD_UPDATE,"","");
 	wait_command_finish();
 	send_message(PK_COMMAND,CMD_LIST,"","");
 	wait_command_finish();
@@ -679,6 +891,22 @@ create_mMain(GtkWidget  *window)
 		mMain_items, NULL);
 	gtk_window_add_accel_group (GTK_WINDOW (window), accelgroup);
 
+	miUpdate = gtk_item_factory_get_item_by_action(itemfactory,
+		MI_PACKAGES_UPDATE);
+	miSysUpgrade = gtk_item_factory_get_item_by_action(itemfactory,
+		MI_PACKAGES_UPGRADE);
+	miSelectLocal = gtk_item_factory_get_item_by_action(itemfactory,
+		MI_FILE_INSTALL);
+	miApply = gtk_item_factory_get_item_by_action(itemfactory,
+		MI_PACKAGES_APPLY);
+	miFilterInst = gtk_item_factory_get_item_by_action(itemfactory,
+		MI_FILTER_INST);
+	miFilterNotInst = gtk_item_factory_get_item_by_action(itemfactory,
+		MI_FILTER_NOTINST);
+	
+	gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(miFilterInst),TRUE); 
+	gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(miFilterNotInst),TRUE); 
+	
 	return (gtk_item_factory_get_widget (itemfactory, "<main>"));
 }
 
@@ -720,9 +948,11 @@ create_fMain (void)
   gtk_container_add(GTK_CONTAINER(fMain),vbox);
 	
   tooltips = gtk_tooltips_new ();
+  
+  sbar = gtk_statusbar_new();
+  gtk_box_pack_end(GTK_BOX(vbox),sbar,FALSE,TRUE,0);
 
   /* main menu */ 
-  
   mMain = create_mMain(fMain);
   gtk_box_pack_start(GTK_BOX(vbox),mMain,FALSE,TRUE,0);
   
@@ -733,25 +963,10 @@ create_fMain (void)
 			       GTK_ORIENTATION_HORIZONTAL);
   gtk_toolbar_set_style (GTK_TOOLBAR (toolbar), GTK_TOOLBAR_ICONS);
 
-/*  bUpdate = gtk_toolbar_insert_stock (GTK_TOOLBAR (toolbar), GTK_STOCK_GO_DOWN, 
-  			_("Update package lists"),	_("Update package lists"),
-			   (GtkSignalFunc) on_packages_update_clicked , NULL, -1);
-			   
-  pw = gtk_image_new_from_pixbuf (gpe_find_icon ("refresh"));
-  bSysUpgrade = gtk_toolbar_append_item (GTK_TOOLBAR (toolbar), _("Update System"),
-			   _("Update System"), _("Update entire system over an internet connection."), pw,
-			   (GtkSignalFunc) on_system_update_clicked , NULL);
-*/
   bApply = gtk_toolbar_insert_stock (GTK_TOOLBAR (toolbar), GTK_STOCK_APPLY,
 			   _("Apply package selection"), _("Apply package selection"),
 			   (GtkSignalFunc) on_package_install_clicked , NULL, -1);
 			   
-/*  pw = gtk_image_new_from_pixbuf (gpe_find_icon ("local-package"));
-  bSelectLocal = gtk_toolbar_append_item (GTK_TOOLBAR (toolbar), _("Install package"),
-			   _("Install package"), 
-			   _("Select a package to install in local filesystem."), pw,
-			   (GtkSignalFunc) on_select_local , NULL);
-*/			   
   gtk_toolbar_append_space(GTK_TOOLBAR(toolbar));
   
   pw = gtk_image_new_from_pixbuf(gpe_find_icon ("exit"));
@@ -768,26 +983,24 @@ create_fMain (void)
 	
   gtk_object_set_data(GTK_OBJECT(notebook),"tooltips",tooltips);
   
-  sbar = gtk_statusbar_new();
-  gtk_box_pack_start(GTK_BOX(vbox),sbar,FALSE,TRUE,0);
-
-  /* list tab */	
+  /* installed tab */	
   vbox = gtk_vbox_new(FALSE,gpe_get_boxspacing());
 
-  cur = gtk_label_new(_("Packages"));
+  cur = gtk_label_new(_("List"));
   gtk_notebook_append_page(GTK_NOTEBOOK(notebook),vbox,cur);
 
   cur = gtk_label_new(NULL);
   gtk_misc_set_alignment(GTK_MISC(cur),0.0,0.5);
-  tmp = g_strdup_printf("<b>%s</b>",_("Package Selection"));
+  tmp = g_strdup_printf("<b>%s</b>",_("Package List"));
   gtk_label_set_markup(GTK_LABEL(cur),tmp);
   free(tmp);
   gtk_box_pack_start(GTK_BOX(vbox),cur,FALSE,TRUE,0);	
 	
   cur = gtk_scrolled_window_new(NULL,NULL);
-  gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(cur),GTK_POLICY_AUTOMATIC,GTK_POLICY_AUTOMATIC);
-  gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW(cur),GTK_SHADOW_IN);
-  gtk_box_pack_start(GTK_BOX(vbox),cur,TRUE,TRUE,0);	
+  gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(cur),
+  	GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
+  gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW(cur), GTK_SHADOW_IN);
+  gtk_box_pack_start(GTK_BOX(vbox), cur, TRUE, TRUE, 0);	
 
   /* packages tree */
   treeview = gtk_tree_view_new_with_model (GTK_TREE_MODEL (store));
@@ -821,7 +1034,8 @@ create_fMain (void)
 							   NULL);
 	gtk_tree_view_column_set_resizable(GTK_TREE_VIEW_COLUMN(column),TRUE);
 	gtk_tree_view_append_column (GTK_TREE_VIEW (treeview), column);
-	
+
+  
   /* messages tab */
   vbox = gtk_vbox_new(FALSE,gpe_get_boxspacing());
 
@@ -836,9 +1050,11 @@ create_fMain (void)
   gtk_box_pack_start(GTK_BOX(vbox),cur,FALSE,TRUE,0);	
 
   cur = gtk_scrolled_window_new(NULL,NULL);
-  gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(cur),GTK_POLICY_AUTOMATIC,GTK_POLICY_AUTOMATIC);
+  gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(cur),
+  	GTK_POLICY_AUTOMATIC,GTK_POLICY_AUTOMATIC);
   gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW(cur),GTK_SHADOW_IN);
-  gtk_tooltips_set_tip (tooltips, cur, _("This window shows all messages from the packet manager."), NULL);
+  gtk_tooltips_set_tip (tooltips, cur, 
+  	_("This window shows all messages from the packet manager."), NULL);
   
   txLog = gtk_text_view_new();
   gtk_text_view_set_wrap_mode(GTK_TEXT_VIEW(txLog),GTK_WRAP_WORD);
